unit NStruct_Header;

interface

type
 // coordinates:
 xyz = array [0 .. 2] of double;

 // node = end of rod where links are connected:
 node = array [0 .. 1] of record
                           p : xyz;
                           fix : longbool; // fixed or free node
                          end;

 // parameters of one rod:
 TRod = record
         s : node;
         lRod : double;
        end;
 TabTRod = array of TRod; // all rods

 // parameters of one link:
 TLnk = record
         // link between rod1 node1 and rod2 node2:
         r1, n1, r2, n2,
         // # of independant module:
         nMod : longword;
         // length of unstretched link:
         lLnk,
         // 1/2 * stiffness of link:
         k05 : double;
        end;
 TabTLnk = array of TLnk; // all links

 // coordinates of one external force vector:
 TForce = array [0 .. 1] of xyz;
 TabTForce = array of TForce; // all external applied forces

 // if Lagrange multipliers are to be reused in successive calculations:
 TabDouble = array of double;

 // "forward" declaration:
 pTSyst = ^TSyst;

 // user defined:
 TExtern = function (syst : pTSyst) : double; stdcall;
 TabTExtern = array of TExtern;

 // the structure to be calculated:
 TSyst = record
          nb, nl : longword;    // # of rods and of links (nb = 32 max in DEMO VERSION)
          rd : TabTRod;         // system of rods
          lk : TabTLnk;         // links between nodes
          fc : TabTForce;       // external forces applied to the system (optional)
          lg : TabDouble;       // user's Lagrange multipliers (optional), MUST be initalized at 0.0 if used
          fe : TabTExtern;      // customed elastic forces (optional, nil in DEMO VERSION)
          ce : TabTExtern;      // external constraints (optional, nil in DEMO VERSION)
          p1, p2 : pointer;     // parameters for fe and ce (optional, nil in DEMO VERSION)
         end;
 
 // callback:
 TCallBack = procedure (sys : pTSyst;
                        opt : plongword); stdcall;

//############################################################################//
//################### F U N C T I O N S  O F  T H E  D L L ###################//
//############################################################################//

// Solve: solves the system of rods and links
// IN: sys = structure to be calculated
// IN: cbk = address of callback procedure (optional)
// OUT: sys = calculated structure
// Return: result of calculation (see NStruct_MsgCodes)
function Solve (sys : pTSyst;
                cbk : TCallback;
                opt : plongword) : longword; stdcall;

// GetHalfStiffnessValue: retrieves default half stiffness value
// IN: none
// OUT: none
// Return: default half stiffness value
function GetHalfStiffnessValue : double; stdcall;

// SetHalfStiffnessValue: changes default half stiffness value
// IN: new default half stiffness value
// OUT: none
// Return: none
// procedure SetHalfStiffnessValue (value : double); stdcall; NOT IN DEMO VERSION

// NukeThemAll: kills all internal loops and threads
// IN: none
// OUT: none
// Return: none
procedure NukeThemAll; stdcall;

// Distance: distance between two points
// IN: coordinates of points
// OUT: none
// Return: calculated distance
function Distance (const p1, p2 : xyz) : double; stdcall;

// Modules: # of independant modules of the system
// IN: structure to be calculated, if ssmod is true: looks for submodules
// OUT: structure to be calculated (links are "tagged")
// Return: # of modules
function Modules (Syst : pTSyst;
                  ssmod : longbool) : longword; stdcall;

// Potential: potential energy of the system
// IN: structure to be calculated
// OUT: none
// Return: potential energy
function Potential (Syst : pTSyst) : double; stdcall;

implementation

function Solve (sys : pTSyst;
                cbk : TCallback;
                opt : plongword) : longword; external 'Structure.dll'; stdcall;

function GetHalfStiffnessValue : double; external 'Structure.dll'; stdcall;

// procedure SetHalfStiffnessValue (value : double); external 'Structure.dll'; stdcall; NOT IN DEMO VERSION

procedure NukeThemAll; external 'Structure.dll'; stdcall;

function Distance (const p1, p2 : xyz) : double; external 'Structure.dll'; stdcall;

function Modules (Syst : pTSyst;
                  ssmod : longbool) : longword; external 'Structure.dll'; stdcall;

function Potential (Syst : pTSyst) : double; external 'Structure.dll'; stdcall;

end.
